
/*
 * QDPF - Quick And Dirty Port Forwarder
 *
 * (C) John Hawksley 2000
 *
 * LICENSING:  See readme.txt, or http://www.hawksley.net/software/qdpf/
 *
 * mail: john at bawue.de
 *
 * REVISION: 1.01
 */

/**
 *
 * AsyncReader is the thread which listens on a DIS for incoming data.
 *
 * When running, the thread attempts to read a byte from the DIS.  If the
 * socket has no data on it, we try to read a byte anyhow.  If no link is
 * available, Java will raise an IOException, which I handle as Exception.
 *
 * This is really stupid.  I wish I could have a non-blocking read
 * (EWOULDBLOCK?) and also some clearer way of working out if a socket
 * is actually connected at the far end.
 *
 * INSTRUCTIONS
 *
 * - If AsyncReader turns blue, data is available, if red - hard luck.
 *
 * - Instantiate and start this thread.
 * - Call isReady() to see if data is available.
 * - Call isShut to see if the connection is closed (but see last point)
 * - Call readData() to get an array of bytes.  The length of this array
 *   depends on how much data was available when the class did the read.
 * - If thread exits, connection has shut.
 */

import java.io.*;
import java.net.*;

class AsyncReader extends Thread
{
	protected DataInputStream r;
	byte[] b;
	boolean dataReady = false;
	boolean shut = false;

	AsyncReader(DataInputStream r, String name)
	{
		super(name);
		this.r = r;
	}

	public boolean isReady()
	{
		return( dataReady );
	}

	public byte[] readData()
	{
		byte[] a = b;
		dataReady = false;
		return a;
	}

	public boolean isShut()
	{
		return shut;
	}

	public void run()
	{

   		 for(;;)
   		 {
			try
			{
	   		 	if( !dataReady && !shut )
	   		 	{
	   		 		try
	   		 		{
	   		 			byte tempByte;

						if(r.available() == 0) // cx might be closed
						{
							b = new byte[1];
							b[0] = r.readByte();
							dataReady = true;
						}
						else
						{
							b = new byte[r.available()];
							r.readFully(b, 0, b.length);
							dataReady = true;
						}
	   		 		}
	   		 		catch(Exception e)
	   		 		{
	   		 			shut = true;
	   		 			return;
	   		 		}
	   		 	}
	   		 	else
	   		 	{
	   		 		if ( shut )
	   		 			return;

	   		 			 this.yield();

	   			}
	   		}
	   		catch(Exception e)
	   		{ System.out.println(e); }
   		 }
	}
}
