
/*
 * QDPF - Quick And Dirty Port Forwarder
 *
 * (C) John Hawksley 2000
 *
 * LICENSING:  See readme.txt, or http://www.hawksley.net/software/qdpf/
 *
 * mail: john at bawue.de
 *
 * REVISION: 1.01
 */

/**
 * This is the thread started for each incoming connection.
 *
 * What happens here is basically that we get In and Out streams for the
 * source connection (Client) and the destination forwarded port (server).
 *
 * This thread then just takes data from the client and passes it to the server,
 * and vice versa.
 *
 * In the words of Pete King, my old X-Windows programming lecturer:
 *   "Simple, Straightforward, trivial, easy!"
 */

import java.io.*;
import java.net.*;

class Connection extends
Thread
{
	Socket clientSocket;
	Socket outSocket;
	DataInputStream clientIn;
	DataOutputStream clientOut;

	DataInputStream serverIn;
	DataOutputStream serverOut;

	public Connection(Socket cs, int op, String sh)
	{
		clientSocket = cs;

		try
		{
			clientIn = new  DataInputStream(clientSocket.getInputStream());
			clientOut = new DataOutputStream(clientSocket.getOutputStream());

			outSocket = new Socket(sh, op);

			serverIn = new  DataInputStream(outSocket.getInputStream());
			serverOut = new DataOutputStream(outSocket.getOutputStream());
		}
		catch(IOException e)
		{
			try
			{
				clientSocket.close();
				outSocket.close();
			}
			catch(IOException e2)
			{
				return;
			}
		}
		this.start();
	}

	public synchronized void run()
	{
		// byte clientInByte, serverInByte;
		AsyncReader clientInR = new AsyncReader(clientIn, "Client");
		AsyncReader serverInR = new AsyncReader(serverIn, "Server");

		clientInR.start();
		serverInR.start();


		try
		{
			for(;;)
			{

				if( clientInR.isReady() )
				{
					byte[] tba = clientInR.readData();
					serverOut.write( tba, 0, tba.length );
				}

				if( serverInR.isReady() )
				{
					byte[] tba = serverInR.readData();
					clientOut.write( tba, 0, tba.length );
				}

				if( clientInR.isShut() || serverInR.isShut() )
				{
					return;
				}

				this.yield();
			}
		}
		catch(IOException e) { ; }
		finally
		{
			try
			{
				clientIn.close();
				outSocket.close();
			}
			catch(IOException e)
			{ ; }
		}
	}
}
